//
// Copyright (c) 2002
// Ronald Kevin Burton
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. This software is provided solely as part of ".NET Common Language
// Runtime Unleashed" to help illustrate concepts and principles described in 
// that book. Any use is at your own risk.
//
// Permission to use or copy this software for any purpose is hereby granted 
// without fee, provided the above notices are retained on all copies.
// Permission to modify the code and to distribute modified code is granted,
// provided the above notices are retained, and a notice that the code was
// modified is included with the above copyright notice.
//
// 
using System;
using System.Diagnostics;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.IO;

namespace LicenseTimeBombClass
{
	/// <summary>
	/// Summary description for TimeBombLicenseProvider.
	/// </summary>
	public class TimeBombLicenseProvider : LicenseProvider
	{
		protected virtual bool IsKeyValid(string key, Type type) 
		{
			// If string isn't empty
			if (key != null) 
			{
				DateTime now = DateTime.Now;
				//Get number of processors license is valid for
				int startIndex = key.IndexOf(',') + 1;
				int endIndex = key.IndexOf(',', startIndex) + 1;
				DateTime validStartTime = new DateTime(Convert.ToInt64(key.Substring(startIndex, endIndex - startIndex - 1)));
				DateTime validEndTime = new DateTime(Int64.Parse(key.Substring(endIndex)));
				
				Debug.WriteLine("Start date: " + validStartTime.ToString() + " End date: " + validEndTime.ToString() + "\n" + "Now: " + now.ToString());

				Debug.WriteLine(string.Format("Now is greater than start: {0}", now > validStartTime));
				Debug.WriteLine(string.Format("Now is less than end: {0}", now < validEndTime));
				//validate license string and number of processors
				return (now > validStartTime && now < validEndTime && key.StartsWith(string.Format("{0} is a licensed component.",type.FullName)));
			}
			return false;
		}

		public override License GetLicense(LicenseContext context, Type type, object instance, bool allowExceptions) 
		{
			TimeBombLicense lic = null;
            
			Debug.Assert(context != null, "No context provided!");
			//if no context is provided, do nothing
			if (context != null) 
			{
				//if this control is in runtime mode
				if (context.UsageMode == LicenseUsageMode.Runtime) 
				{
					//retrieve the stored license key
					string key = context.GetSavedLicenseKey(type, null);

					//Check if the stored license key is null
					//and call IsKeyValid to make sure the
                                                                        //license key is valid
					if (key != null && IsKeyValid(key, type)) 
					{
						//if the key is valid create a new license
						lic = new TimeBombLicense(this, key);
					}
				}

				//if we're in design mode or 
				//a suitable license key wasn't found in 
				//the runtime context,
				//attempt to look for a .LIC file
				if (lic == null) 
				{
					//build up the path where the .LIC file
					//should be
					string modulePath = null;

					// try and locate the file for the assembly
					if (context != null) 
					{
						ITypeResolutionService resolver = (ITypeResolutionService)context.GetService(typeof(ITypeResolutionService));
						if (resolver != null)
							modulePath = resolver.GetPathOfAssembly(type.Assembly.GetName());
					}


					if (modulePath == null)
						modulePath = type.Module.FullyQualifiedName;

					//get the path from the file location
					string moduleDir = Path.GetDirectoryName(modulePath);
					
					//build the path of the .LIC file
					string licenseFile = moduleDir + "\\" + type.FullName + ".lic";

					Debug.WriteLine("Path of license file: " + licenseFile);

					//if the .LIC file exists, dig into it
					if (File.Exists(licenseFile)) 
					{
						//crack the file and get the first line
						Stream licStream = new FileStream(licenseFile, FileMode.Open, FileAccess.Read, FileShare.Read);
						StreamReader sr = new StreamReader(licStream);
						string s = sr.ReadLine();
						sr.Close();

						Debug.WriteLine("Contents of license file: " + s);

						//check if the key is valid
						if (IsKeyValid(s, type)) 
						{
							//valid key so create a new License
							lic = new TimeBombLicense(this, s);
						}
					}

					//If we managed to create a license, put it into the context.
					if (lic != null) 
					{
						context.SetSavedLicenseKey(type, lic.LicenseKey);
					}
				}

			}
			return lic;
		}
		public class TimeBombLicense : License 
		{
			private TimeBombLicenseProvider owner;
			private string key;
			private DateTime validStartTime;
			private DateTime validEndTime;

			public TimeBombLicense(TimeBombLicenseProvider owner, string key) 
			{
				this.owner = owner;
				this.key = key;
				int startIndex = key.IndexOf(',') + 1;
				int endIndex = key.IndexOf(',', startIndex) + 1;
				this.validStartTime = new DateTime(Convert.ToInt64(key.Substring(startIndex, endIndex - startIndex - 1)));
				this.validEndTime = new DateTime(Int64.Parse(key.Substring(endIndex)));
			}
			public override string LicenseKey 
			{ 
				get 
				{
					return key;
				}
			}

			public DateTime ValidStartTime 
			{
				get 
				{
					return validStartTime;
				}
			}
			public DateTime ValidEndTime 
			{
				get 
				{
					return validEndTime;
				}
			}

			public override void Dispose() 
			{
			}
		}
	}
}
